<?php

namespace InSegment\WorkersFramework\Drivers\RabbitMq;

use Shavshukov\EventAppBase\Service\ErrorReport;
use React\EventLoop\LoopInterface;
use React\Promise\PromiseInterface;
use Bunny\Async\Client as AsyncClient;
use Bunny\Client;

use InSegment\WorkersFramework\Interfaces\QueueConnectorInterface;

class Connector implements QueueConnectorInterface
{
    /**
     * Memory queue channel
     *
     * @var \Bunny\Channel
     */
    protected $channel;
    
    /**
     * Connection options
     * 
     * @var \InSegment\WorkersFramework\Drivers\RabbitMq\ConnectionOptions
     */
    protected $options;
    
    /**
     * Constructor
     * 
     * @param \InSegment\WorkersFramework\Drivers\RabbitMq\ConnectionOptions $connectionOptions
     */
    public function __construct(ConnectionOptions $connectionOptions)
    {
        $this->options = $connectionOptions;
    }
    
    /**
     * Get prefetch size from application config
     * 
     * @return int
     */
    public function getPrefetchSize(): int
    {
        return $this->options->get('prefetch_size', 0);
    }
    
    /**
     * Get prefetch count from application config
     * 
     * @return int
     */
    public function getPrefetchCount(): int
    {
        return $this->options->get('prefetch_count', 1);
    }
    
    /**
     * Create async driver
     * 
     * @param \React\EventLoop\LoopInterface $loop
     * @param bool $sync
     * @return \React\Promise\PromiseInterface<\InSegment\WorkersFramework\Drivers\RabbitMq\QueueDriverInterface>
     */
    public function createDriver(LoopInterface $loop): PromiseInterface
    {
        return (new AsyncClient($loop, $this->options->getClientOptions()))->connect()->then(function (AsyncClient $client) {
            return AsyncChannel::create($this, $client->channel());
        }, function (\Exception $e) {
            ErrorReport::report($e, true);
            exit(-1);
        });
    }
    
    /**
     * Create sync channel driver
     * 
     * @return \Bunny\Channel
     */
    public function createSyncDriver()
    {
        $bunny = new Client($this->options->getClientOptions());
        $bunny->connect();
        return $bunny->channel();
    }
}
