<?php

namespace InSegment\WorkersFramework\Interfaces;

interface DeclareQueueBuilderInterface
{
    
    /**
     * Set name of the queue to be declared
     * 
     * @param string $queueName
     * @return \InSegment\WorkersFramework\Interfaces\DeclareQueueBuilderInterface
     */
    public function setName(string $queueName): DeclareQueueBuilderInterface;
    
    /**
     * Set name of the queue to move the messages, which were not acquired from that queue
     * 
     * @param string $dlxQueueName
     * @return \InSegment\WorkersFramework\Interfaces\DeclareQueueBuilderInterface
     */
    public function setDlxQueueName(string $dlxQueueName): DeclareQueueBuilderInterface;
    
    /**
     * Set ttl of the message before it would be deleted or moved to dlx queue
     * 
     * @param int $messageTtl
     * @return \InSegment\WorkersFramework\Interfaces\DeclareQueueBuilderInterface
     */
    public function setMessageTtl(int $messageTtl): DeclareQueueBuilderInterface;
    
    /**
     * Finish building and declare the queue and get count of the messages
     * 
     * @param bool $delete
     * @return \React\Promise\PromiseInterface<int>
     */
    public function doDeclare(bool $delete = false): \React\Promise\PromiseInterface;
    
}
