<?php

namespace InSegment\WorkersFramework\Models;

use React\ChildProcess\Process;
use Carbon\Carbon;

class WorkerPidKiller
{

    /**
     * Kill previous processes PIDs
     * 
     * @param \Shavshukov\EventAppBase\ApplicationInterface $app
     */
    public function killPreviousPids($app)
    {
        $lockfile = $app->config('worker_pid_lockfile');
        if (!$lockfile || !file_exists($lockfile) || !is_readable($lockfile)) {
            echo "Warning: workers PID lock file does not exist!\n";
            return;
        }
        if (!preg_match('~wtmp begins (.+)~', shell_exec('last reboot -1'), $matches)) {
            echo "Warning: can not get the last reboot time!\n";
            return;
        }

        try {
            $rebootDateTime = Carbon::parse($matches[1]);
            $fileDateTime = Carbon::createFromTimestamp(filectime($lockfile));
        } catch (\Exception $e) {
            echo "Warning: can not parse the dates: {$e->getTraceAsString()}\n";
            return;
        }

        if ($rebootDateTime->greaterThan($fileDateTime)) {
            echo "Deleting the lock file after system reboot\n";
            unlink($lockfile);
            return;
        }

        if (($handle = fopen($lockfile, 'r'))) {
            while (($pid = fgets($handle)) !== false) {
                echo "Killing process PID {$pid}";
                (new Process("kill {$pid}"))->start($app->getLoop());
            }
            fclose($handle);
            unlink($lockfile);
        }
    }
}
