<?php

namespace InSegment\WorkersFramework\Queues;

use Shavshukov\EventAppBase\Interfaces\ModelInterface;

use InSegment\WorkersFramework\Interfaces\DeclareQueueBuilderInterface;
use InSegment\WorkersFramework\Interfaces\ConnectionListInterface;
use InSegment\WorkersFramework\Interfaces\QueueDriverInterface;
use InSegment\WorkersFramework\Interfaces\QueueInterface;

class ConnectionsList implements ConnectionListInterface
{
    /**
     * Connections config
     *
     * @var \InSegment\WorkersFramework\Queues\ConnectionsConfig 
     */
    protected $config;
    
    /**
     * Established connections
     *
     * @var \InSegment\WorkersFramework\Interfaces\QueueDriverInterface[]
     */
    protected $connections = [];
    
    /**
     * Queues by their names
     *
     * @var \InSegment\WorkersFramework\Interfaces\QueueInterface;[]
     */
    protected $queues = [];
    
    /**
     * Constructor
     * 
     * @param \InSegment\WorkersFramework\Queues\ConnectionsConfig $config
     * @param array $list \InSegment\WorkersFramework\Interfaces\QueueDriverInterface[]
     */
    public function __construct(ConnectionsConfig $config, array $list)
    {
        $this->config = $config;
        
        foreach ($list as $name => $connector) {
            $this->addConnection($name, $connector);
        }
    }

    /**
     * {@inheritDoc}
     */
    public function getConnection(string $connectionName): QueueDriverInterface
    {
        if (!isset($this->connections[$connectionName])) {
            throw new \Exception("Connection {$connectionName} is not connected");
        }
        
        return $this->connections[$connectionName];
    }
    
    /**
     * {@inheritDoc}
     */
    public function getQueue(string $queueName): QueueInterface
    {
        if (!isset($this->queues[$queueName])) {
            throw new \Exception("Queue {$queueName} is not mapped to any connection");
        }
        
        return $this->queues[$queueName];
    }
    
    /**
     * {@inheritDoc}
     */
    public function declareQueue(string $queueName): DeclareQueueBuilderInterface
    {
        return $this->getQueue($queueName)->getConnection()->declare()->setName($queueName);
    }
    
    /**
     * Add connection to the list
     * 
     * @param string $connectionName
     * @param \InSegment\WorkersFramework\Interfaces\QueueDriverInterface $connection
     * @return null
     */
    protected function addConnection(string $connectionName, QueueDriverInterface $connection)
    {
        $options = $this->config->getConnectionOptions($connectionName);
        $this->connections[$connectionName] = $connection;
        $this->addConnectionQueues($connection, $options, $connectionName);
    }
    
    /**
     * Add mapped queues
     * 
     * @param \InSegment\WorkersFramework\Interfaces\QueueDriverInterface $connection
     * @param \Shavshukov\EventAppBase\Interfaces\ModelInterface $connectionOptions
     */
    protected function addConnectionQueues(QueueDriverInterface $connection, ModelInterface $connectionOptions)
    {
        foreach ($connectionOptions->get('queues') as $queueName) {
            $this->queues[$queueName] = new Queue($connection, $queueName);
        }
    }
    
}
