<?php

namespace Workers;

use PDO;
use InSegment\WorkersFramework\Services\Queue;

abstract class AbstractWorker
{
    private $db;
    private $config;
    public $dbName;
    public $key;

    public function __construct(string $key, array $config = [])
    {
        $this->config = $config;
        $this->dbName = 'app_system';
        $this->key = $key;
    }

    public function getConfig()
    {
        return $this->config;
    }

    public function getDatabaseConnection(): PDO
    {
        if (!isset($this->db)) {
            $config = $this->getConfig();
            $options = [
                PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
            ];
            $this->db = new PDO(
                "mysql:host={$config['db_host']};port={$config['db_port']};dbname={$config['db_name']};charset=utf8",
                $config['db_user'],
                $config['db_pass'],
                $options
            );
        }

        return $this->db;
    }

    public function publishQueueMessage(string $queueName, string $message)
    {
        return Queue::publishMessage($queueName, $message);
    }

    abstract public function run($message);
}
